globals
[
  grid-x-inc         ;; the amount of patches in between two roads in the x direction
  grid-y-inc         ;; the amount of patches in between two roads in the y direction

  roads              ;; agentset containing the patches that are roads
  highway            ;; represented by one grey patch where cars are going twice as faster than in the city
  chargers           ;; agentset of chargers


  EVs-not-get-home  ;; counter to monitor how many EVs failed to get home at each day
  Avg-EVs-not-get-home-a-day ;; making average from all days
  EVs-with-0-energy  ;; how many EVs are without energy (stopped)
  Avg-km-made-EV-a-day  ;; avg km made by one EV
  Max-km-made-EV-a-day  ;; max km made by one EV
  Min-km-made-EV-a-day  ;; min km made by one EV
  Avg-km-made-ICEV-a-day ;; avg km made by one ICEV
  Max-km-made-ICEV-a-day  ;; max km made by one ICEV
  Min-km-made-ICEV-a-day  ;; min km made by one ICEV
  Ticks-for-one-km-made-EV ;; how many ticks on average it takes to EV (active time + charging time both public and home)
  Ticks-for-one-km-made-ICEV  ;; how many ticks on average it takes to ICEV (active time)
  CZK-invested-for-1-km-EV  ;; average money invested into 1km made for EV (price, maintenance, electricity)
  CZK-invested-for-1-km-ICEV  ;; average money invested into 1km made for ICEV (price, maintenance, gasoline)
  CZK-invested-for-1-day-EV  ;; average money invested daily EV
  CZK-invested-for-1-day-ICEV  ;; average money invested daily ICEV
  Avg-hours-at-home-a-day-ICEV ;; average hours at home spend in the end of the day for ICEV
  Avg-hours-at-home-a-day-EV ;; average hours at home spend in the end of the day for EV

  Min-hours-at-home-a-day-EV  ;; min of hours a day one EV spends at home
  Min-hours-at-home-a-day-ICEV  ;; min of hours a day one ICEV spends at home
  Max-hours-at-home-a-day-EV  ;; max of hours a day one EV spends at home
  Max-hours-at-home-a-day-ICEV  ;; max of hours a day one ICEV spends at home
  Km-made-by-EV-total  ;; total km made by average EV
  Km-made-by-ICEV-total  ;; total km made by average ICEV
  Avg-active-hours-EV-a-day  ;; for calibration
  Max-active-hours-EV-a-day  ;; for calibration
]

breed [ICEVs ICEV]   ;; 2 kinds of turtles
breed [EVs EV]


turtles-own
[
  energy    ;; energy to move (electricity)
  invested  ;; money invested (price + energy + maintenance)
  work1     ;; the patch where they work -> first goal
  work2     ;; second goal
  house     ;; the patch where they live
  goal      ;; where the turtle is currently headed
  home-charger? ;; if an EV has the charger at home -> it will charge during the rest of the day after it has done its trip
  another-city? ;; if car is comuting between 2 cities -> house is in different city than work1 and work2
  choices   ;; possible roads where to move-on when car faces next-patch
  choice    ;; reported when facing next-patch to achieve goal (house/work)
  count-down ;; timer on a highway to represent moving to another city (= how many km highway has)
  worked-today? ;; if car has already been at work1 today
  worked2-today?  ;; if car has already been to work2 today
  big-charged?  ;; if car has already been on a big charger on a highway
  trip-distance-total  ;; approximate expected distance of whole day trip, depends if car is comuting through highway or not, it can differ if car has to search for chargers (when need-charge? = true)
  trip-distance  ;; distance from now achieved goal to the next goal (work1, work2, highway, highway charger, house)
  need-charge?  ;; when car is starting its tour to another goal it has to decide if it has enough energy for the journey, if not it has to go to a charger first
  two-goals?  ;; when car has to go to work1 and work2 right after in the same day
  highway-distance  ;; own distance between 2 cities when commuting, limited by max-distance-between-two-cities
  got-home?  ;; to determine if arrived at home today
  distance-group  ;; group assigned based on highway distance
  goal-group ;; 1 if having 1 work and 2 if having 2 works (goals)
  highway-done? ;; as big-charged but for ICEV

  km-made-by-EV   ;; km made by the specific EV
  km-made-by-ICEV  ;; km made by the specific ICEV
  total-active-ticks-ICEV  ;; ticks when vehicle was moving
  total-active-ticks-EV  ;; ticks when vehicle was moving
  total-charging-ticks  ;; total ticks spend by charging of EV
  home-time-ticks-EV    ;; total time spent at home for EV
  home-time-ticks-ICEV  ;; total time spent at home for ICEV
  ]


;;;;;;;;;;;;;;;;
;; SETUP MAIN ;;
;;;;;;;;;;;;;;;;

to setup
  clear-all
  resize-world (world-size * -1)  world-size (world-size * -1) world-size   ;; set world size by slider world-size -> always a square


  setup-patches  ;; viz SETUP Procedures

  let goal-candidates patches with
  [pcolor = brown and (any? neighbors with [ pcolor = white ]) and (not any? neighbors with [pcolor = grey]) and (not any? neighbors with [pcolor = blue]) ]
  ;; these brown patches (next to the road and not near highway) can be chosen by a car when choosing its own house and work(s)

  set-default-shape turtles "car"



  create-EVs (number-of-cars * EV-market-share)   ;; create EVs, I choose initial market share with slider EV-market-share
  [
  setup-EVs  ;; viz SETUP Procedures
  set house one-of goal-candidates
  set work1 one-of goal-candidates with [ self != [ house ] of myself ]  ;; chosing own house and work1 and not to be on the same place
  set work2 one-of goal-candidates with [ self != [ house ] of myself  and self != [ work1 ] of myself]  ;; chosing own house and work1 and work2 and not to be on the same place
  move-to house  ;; I start at my house
   ]



  create-ICEVs (number-of-cars - count EVs)  ;; create ICEVs
  [
  setup-ICEVs   ;; viz SETUP Procedures
  set house one-of goal-candidates
  set work1 one-of goal-candidates with [ self != [ house ] of myself ]   ;; chosing own house and work1 and not to be on the same place
  set work2 one-of goal-candidates with [ self != [ house ] of myself  and self != [ work1 ] of myself]  ;; chosing own house and work1 and work2 and not to be on the same place
  move-to house  ;; I start at my house
   ]



  setup-home-chargers ;; viz SETUP Procedures
  setup-two-goals  ;; viz SETUP Procedures, set groups for highway and city distribution
  setup-another-city  ;; viz SETUP Procedures, set going to another city
  setup-highway-distance-EV ;; viz SETUP Procedures, set highway distribution according to histogram
  setup-highway-distance-ICEV  ;; viz SETUP Procedures





  ask turtles [

    if another-city? = false and two-goals? = false;; if not going through highway and having only work1
  [set trip-distance-total
   2 * (distance (patch ([pxcor] of work1) ([pycor] of house)))
   + 2 * (distance (patch ([pxcor] of house) ([pycor] of work1)))  ;; measure distance in patches on the road from house to work1 and back if not comuting between 2 cities
   set goal work1]  ;; I am not going through highway but straight to work1

   if another-city? = false and two-goals? = true;; if not going through highway and having both works
  [set trip-distance-total
   (distance (patch ([pxcor] of work1) ([pycor] of house)))
   + (distance (patch ([pxcor] of house) ([pycor] of work1)))  ;; measure distance in patches on the road from house to work1 and back if not comuting between 2 cities
   + (distance (patch ([pxcor] of work1) ([pycor] of work2)))
   + (distance (patch ([pxcor] of work2) ([pycor] of work1)))  ;; distance from work1 to work2
   + (distance (patch ([pxcor] of work2) ([pycor] of house)))
   + (distance (patch ([pxcor] of house) ([pycor] of work2)))  ;; distance from work2 to house
   set goal work1]  ;; I am not going through highway but straight to work1

  if another-city? = true and two-goals? = false  ;; going through highway and only work1
  [ set trip-distance-total
    2 * distance (patch ([pxcor] of highway) ([pycor] of house))
   + 2 * distance (patch ([pxcor] of house) ([pycor] of highway))  ;; from house to highway and back
   + 2 * highway-distance  ;; highway length
   + 2 * [distance highway] of (patch ([pxcor] of highway) ([pycor] of work1))
  + 2 * [distance highway] of (patch ([pxcor] of [work1] of self) ([pycor] of highway))  ;; from highway to work1 -> distance in patches on the road from house to work1 if comuting between 2 cities
  set goal highway]  ;; I am going through highway

  if another-city? = true and two-goals? = true  ;; going through highway and both works
  [ set trip-distance-total
    2 * distance (patch ([pxcor] of highway) ([pycor] of house))
   + 2 * distance (patch ([pxcor] of house) ([pycor] of highway))  ;; from house to highway
   + 2 * highway-distance  ;; highway length
   + [distance highway] of (patch ([pxcor] of highway) ([pycor] of work1))
  + [distance highway] of (patch ([pxcor] of [work1] of self) ([pycor] of highway))  ;; from highway to work1 -> distance in patches on the road from house to work1 if comuting between 2 cities
  + (distance (patch ([pxcor] of work1) ([pycor] of work2)))
  + (distance (patch ([pxcor] of work2) ([pycor] of work1)))  ;; distance from work1 to work2
  + (distance (patch ([pxcor] of highway) ([pycor] of work2)))
  + (distance (patch ([pxcor] of work2) ([pycor] of highway)))  ;; distance from work2 to highway
  set goal highway]]  ;; I am going through highway



  ask EVs [

    set trip-distance  ;; to determine if need-charge?
    (distance (patch ([pxcor] of goal) ([pycor] of house)))
    + (distance (patch ([pxcor] of house) ([pycor] of goal)))  ;; distance only to work1

    if another-city? = true ;; if first goal is the highway
     [set trip-distance
      (distance (patch ([pxcor] of highway) ([pycor] of house)))
    + (distance (patch ([pxcor] of house) ([pycor] of highway)))
    + ( highway-distance / 2 ) ] ;; need to add distance to the middle of the highway where the violet charger is

    if trip-distance > (energy - 20)   ;; calculate if I have enough energy for this part-trip + 20km reserve
    [set need-charge? true  ;; I have to charge during this part-trip
     set goal min-one-of chargers [distance myself]]  ;; go first to a charger
    ]

set EVs-not-get-home 0  ;; reset counter for failed returns home

display-labels    ;; for displaying energy left of EVs
reset-ticks

end

;;;;;;;;;;;;;;;;;;;;;;
;; SETUP Procedures ;;
;;;;;;;;;;;;;;;;;;;;;;


to setup-patches
  ask patches
  [set pcolor brown]
  ;; create brown patches where houses and works will be located

  set roads patches with [
    (pxcor = min-pxcor) or (pxcor = min-pxcor + 5) or (pxcor = min-pxcor + 10) or (pxcor = min-pxcor + 15) or (pxcor = min-pxcor + 20) or (pxcor = min-pxcor + 25) or (pxcor = max-pxcor) or
    (pycor = min-pycor) or (pycor = min-pycor + 5) or (pycor = min-pycor + 10) or (pycor = min-pycor + 15) or (pycor = min-pycor + 20) or (pycor = min-pycor + 25) or (pycor = max-pycor)
  ]
  ;; make roads based on setup-globals
  ask roads [set pcolor white]  ;; make roads visible

ask n-of number-of-chargers roads [set pcolor blue ]  ;; randomly create chargers on roads based on slider: number-of-chargers
ask one-of roads with [pxcor = (world-size - 1) and pycor = (world-size * -1)] [set pcolor blue]
ask one-of roads with [pxcor = (world-size) and pycor = (world-size * -1 + 1)] [set pcolor blue]  ;; make chargers on both ends of highway
set chargers roads with [pcolor = blue]  ;; give name to chargers

ask one-of roads with [pxcor = (world-size) and pycor = (world-size * -1)] [set pcolor grey]  ;; make road in the downward-left corner grey to make highway
set highway one-of roads with [pcolor = grey]  ;; make highway
ask patch (world-size - 1 ) ((world-size * -1) + 1) [set pcolor violet ]  ;; make one charger "in the middle" of a highway for EVs
end

to setup-EVs
  set invested price-EVs  ;; initial price chosen by slider
  set km-made-by-EV 0.1  ;; for avoiding an error in monitors
  set color green
  set another-city? false  ;; by default
  set worked-today? false  ;; not been in a work1 yet
  set worked2-today? false  ;; not been in a work2 yet
  set two-goals? false  ;; by default
  set goal-group 1 ;; by default, can be changed by probability of having 2 goals
  set big-charged? false  ;; not on a highway yet

  set got-home? false  ;; for counting failures when not get back home from work
  set energy 164 ;; driving distance for one full battery
  set label-color black  ;; for displaying energy
  set home-charger? false  ;; by default
  set need-charge? false  ;; by default
end

to setup-ICEVs
  set invested price-ICEVs ;; initial price chosen by slider
  set km-made-by-ICEV 0.1  ;; for avoiding the error in monitors
  set color red
  set another-city? false  ;; both goals in the same city by defalut
  set worked-today? false  ;; not been in a work1 yet
  set worked2-today? false  ;; not been in a work2 yet
  set two-goals? false  ;; by default
  set goal-group 1 ;; by default, can be changed by probability of having 2 goals
  set highway-done? false ;; by default
end

to setup-home-chargers
  ask n-of (Probability-of-home-charging * (count EVs)) EVs
 [set home-charger? true]
end
;; setup by slider how many EVs will have charger at home and charge every day after their trip

to setup-two-goals
  ask n-of (Probability-of-having-2-goals-EV * count EVs) EVs
[set two-goals? true
 set goal-group 2]
  ask n-of (Probability-of-having-2-goals-ICEV * count ICEVs) ICEVs
[set two-goals? true
 set goal-group 2]
  end
;; setup how many cars will have two goals

to setup-another-city
  ask n-of (Probability-of-going-to-another-city-1-goal * count EVs with [goal-group = 1]) EVs with [goal-group = 1]
[
  set another-city? true  ;; through highway every time
  set distance-group 100  ;; to set base group for distance distribution
  set count-down 0   ;; counter representing km made on a highway (not been there yet)
 ] ;; share of 0.5 of EVs with 1 goal will commute based on distance distribution

  ask n-of (Probability-of-going-to-another-city-2-goals * count EVs with [goal-group = 2]) EVs with [goal-group = 2]
[
  set another-city? true  ;; through highway every time
  set distance-group 200  ;; to set base group for distance distribution
  set count-down 0   ;; counter representing km made on a highway (not been there yet)
 ] ;; share of 0.4 of EVs with 2 goals will commute based on distance distribution

  ask n-of (Probability-of-going-to-another-city-1-goal * count ICEVs with [goal-group = 1]) ICEVs with [goal-group = 1]
[
  set another-city? true  ;; through highway every time
  set distance-group 100  ;; to set base group for distance distribution
  set count-down 0   ;; counter representing km made on a highway (not been there yet)
 ] ;; share of 0.5 of ICEVs with 1 goal will commute based on distance distribution

  ask n-of (Probability-of-going-to-another-city-2-goals * count ICEVs with [goal-group = 2]) ICEVs with [goal-group = 2]
[
  set another-city? true  ;; through highway every time
  set distance-group 200  ;; to set base group for distance distribution
  set count-down 0   ;; counter representing km made on a highway (not been there yet)
 ] ;; share of 0.4 of ICEVs with 2 goals will commute based on distance distribution

end
;; setup how many cars will commute between 2 cities from the two groups (1 goal and 2 goals)



to setup-highway-distance-EV ;; highway distance distribution for 1 and 2 group

  ;;;;;;GROUP 1 EV;;;;;;;
  ask n-of round (0.015 * count EVs with [another-city? = true and goal-group = 1]) (EVs with [distance-group = 100])
 [set highway-distance random (200 - 151 + 1) + 151
  set distance-group 11]
  ask n-of round (0.03 * count EVs with [another-city? = true and goal-group = 1]) (EVs with [distance-group = 100])
 [set highway-distance random (150 - 101 + 1) + 101
  set distance-group 10]
  ask n-of round (0.045 * count EVs with [another-city? = true and goal-group = 1]) (EVs with [distance-group = 100])
 [set highway-distance random (100 - 91 + 1) + 91
  set distance-group 9]
 ask n-of round (0.06 * count EVs with [another-city? = true and goal-group = 1]) (EVs with [distance-group = 100])
 [set highway-distance random (90 - 81 + 1) + 81
  set distance-group 8]
 ask n-of round (0.075 * count EVs with [another-city? = true and goal-group = 1]) (EVs with [distance-group = 100])
 [set highway-distance random (80 - 71 + 1) + 71
  set distance-group 7]
 ask n-of round (0.09 * count EVs with [another-city? = true and goal-group = 1]) (EVs with [distance-group = 100])
 [set highway-distance random (70 - 61 + 1) + 61
  set distance-group 6]
 ask n-of round (0.105 * count EVs with [another-city? = true and goal-group = 1]) (EVs with [distance-group = 100])
 [set highway-distance random (60 - 51 + 1) + 51
  set distance-group 5]
 ask n-of round (0.12 * count EVs with [another-city? = true and goal-group = 1]) (EVs with [distance-group = 100])
 [set highway-distance random (50 - 41 + 1) + 41
  set distance-group 4]
 ask n-of round (0.135 * count EVs with [another-city? = true and goal-group = 1]) (EVs with [distance-group = 100])
 [set highway-distance random (40 - 31 + 1) + 31
  set distance-group 3]
 ask n-of round (0.15 * count EVs with [another-city? = true and goal-group = 1]) (EVs with [distance-group = 100])
 [set highway-distance random (30 - 21 + 1) + 21
  set distance-group 2]
 ask EVs with [distance-group = 100] ;; rest is about 17.5%, to make sure there is no EV left with group 100
 [set highway-distance random (20 - 10 + 1) + 10
  set distance-group 1]


  ;;;;;;GROUP 2 EV;;;;;;;
   ask n-of round (0.015 * count EVs with [another-city? = true and goal-group = 2]) (EVs with [distance-group = 200])
 [set highway-distance random (200 - 151 + 1) + 151
  set distance-group 11]
  ask n-of round (0.03 * count EVs with [another-city? = true and goal-group = 2]) (EVs with [distance-group = 200])
 [set highway-distance random (150 - 101 + 1) + 101
  set distance-group 10]
  ask n-of round (0.045 * count EVs with [another-city? = true and goal-group = 2]) (EVs with [distance-group = 200])
 [set highway-distance random (100 - 91 + 1) + 91
  set distance-group 9]
 ask n-of round (0.06 * count EVs with [another-city? = true and goal-group = 2]) (EVs with [distance-group = 200])
 [set highway-distance random (90 - 81 + 1) + 81
  set distance-group 8]
 ask n-of round (0.075 * count EVs with [another-city? = true and goal-group = 2]) (EVs with [distance-group = 200])
 [set highway-distance random (80 - 71 + 1) + 71
  set distance-group 7]
 ask n-of round (0.09 * count EVs with [another-city? = true and goal-group = 2]) (EVs with [distance-group = 200])
 [set highway-distance random (70 - 61 + 1) + 61
  set distance-group 6]
 ask n-of round (0.105 * count EVs with [another-city? = true and goal-group = 2]) (EVs with [distance-group = 200])
 [set highway-distance random (60 - 51 + 1) + 51
  set distance-group 5]
 ask n-of round (0.12 * count EVs with [another-city? = true and goal-group = 2]) (EVs with [distance-group = 200])
 [set highway-distance random (50 - 41 + 1) + 41
  set distance-group 4]
 ask n-of round (0.135 * count EVs with [another-city? = true and goal-group = 2]) (EVs with [distance-group = 200])
 [set highway-distance random (40 - 31 + 1) + 31
  set distance-group 3]
 ask n-of round (0.15 * count EVs with [another-city? = true and goal-group = 2]) (EVs with [distance-group = 200])
 [set highway-distance random (30 - 21 + 1) + 21
  set distance-group 2]
 ask EVs with [distance-group = 200] ;; rest is about 17.5%, to make sure there is no EV left with group 100
 [set highway-distance random (20 - 10 + 1) + 10
  set distance-group 1]
end


to setup-highway-distance-ICEV ;; highway distance distribution

  ;;;;;;GROUP 1 ICEV;;;;;;;
    ask n-of round (0.0095 * count ICEVs with [another-city? = true and goal-group = 1]) (ICEVs with [distance-group = 100])
 [set highway-distance random (370 - 301 + 1) + 301 ;; make 1% of ICEVs using the highway to chose highway distance between 300 and max
  set distance-group 14] ;; set group in order not to recieve any other distance from later groups
  ask n-of round (0.019 * count ICEVs with [another-city? = true and goal-group = 1]) (ICEVs with [distance-group = 100])
 [set highway-distance random (300 - 251 + 1) + 251 ;; make 1% of ICEVs using the highway to chose highway distance between 300 and max
  set distance-group 13] ;; set group in order not to recieve any other distance from later groups
  ask n-of round (0.0285 * count ICEVs with [another-city? = true and goal-group = 1]) (ICEVs with [distance-group = 100])
 [set highway-distance random (250 - 201 + 1) + 201 ;; make 2.2% of ICEVs using the highway to chose highway distance between 300 and 250
  set distance-group 12] ;; set group in order not to recieve any other distance from later groups
  ask n-of round (0.038 * count ICEVs with [another-city? = true and goal-group = 1]) (ICEVs with [distance-group = 100])
 [set highway-distance random (200 - 151 + 1) + 151
  set distance-group 11]
  ask n-of round (0.0475 * count ICEVs with [another-city? = true and goal-group = 1]) (ICEVs with [distance-group = 100])
 [set highway-distance random (150 - 101 + 1) + 101
  set distance-group 10]
  ask n-of round (0.057 * count ICEVs with [another-city? = true and goal-group = 1]) (ICEVs with [distance-group = 100])
 [set highway-distance random (100 - 91 + 1) + 91
  set distance-group 9]
 ask n-of round (0.0665 * count ICEVs with [another-city? = true and goal-group = 1]) (ICEVs with [distance-group = 100])
 [set highway-distance random (90 - 81 + 1) + 81
  set distance-group 8]
 ask n-of round (0.076 * count ICEVs with [another-city? = true and goal-group = 1]) (ICEVs with [distance-group = 100])
 [set highway-distance random (80 - 71 + 1) + 71
  set distance-group 7]
 ask n-of round (0.0855 * count ICEVs with [another-city? = true and goal-group = 1]) (ICEVs with [distance-group = 100])
 [set highway-distance random (70 - 61 + 1) + 61
  set distance-group 6]
 ask n-of round (0.095 * count ICEVs with [another-city? = true and goal-group = 1]) (ICEVs with [distance-group = 100])
 [set highway-distance random (60 - 51 + 1) + 51
  set distance-group 5]
 ask n-of round (0.1045 * count ICEVs with [another-city? = true and goal-group = 1]) (ICEVs with [distance-group = 100])
 [set highway-distance random (50 - 41 + 1) + 41
  set distance-group 4]
 ask n-of round (0.114 * count ICEVs with [another-city? = true and goal-group = 1]) (ICEVs with [distance-group = 100])
 [set highway-distance random (40 - 31 + 1) + 31
  set distance-group 3]
 ask n-of round (0.1235 * count ICEVs with [another-city? = true and goal-group = 1]) (ICEVs with [distance-group = 100])
 [set highway-distance random (30 - 21 + 1) + 21
  set distance-group 2]
 ask ICEVs with [distance-group = 100] ;; rest is about 13.55%, to make sure there is no ICEV left with group 100
 [set highway-distance random (20 - 10 + 1) + 10
  set distance-group 1]


  ;;;;;;GROUP 2 ICEV;;;;;;;
  ask n-of round (0.0095 * count ICEVs with [another-city? = true and goal-group = 2]) (ICEVs with [distance-group = 200])
 [set highway-distance random (370 - 301 + 1) + 301 ;; make 1% of ICEVs using the highway to chose highway distance between 300 and max
  set distance-group 14] ;; set group in order not to recieve any other distance from later groups
  ask n-of round (0.019 * count ICEVs with [another-city? = true and goal-group = 2]) (ICEVs with [distance-group = 200])
 [set highway-distance random (300 - 251 + 1) + 251 ;; make 1% of ICEVs using the highway to chose highway distance between 300 and max
  set distance-group 13] ;; set group in order not to recieve any other distance from later groups
  ask n-of round (0.0285 * count ICEVs with [another-city? = true and goal-group = 2]) (ICEVs with [distance-group = 200])
 [set highway-distance random (250 - 201 + 1) + 201 ;; make 2.2% of ICEVs using the highway to chose highway distance between 300 and 250
  set distance-group 12] ;; set group in order not to recieve any other distance from later groups
  ask n-of round (0.038 * count ICEVs with [another-city? = true and goal-group = 2]) (ICEVs with [distance-group = 200])
 [set highway-distance random (200 - 151 + 1) + 151
  set distance-group 11]
  ask n-of round (0.0475 * count ICEVs with [another-city? = true and goal-group = 2]) (ICEVs with [distance-group = 200])
 [set highway-distance random (150 - 101 + 1) + 101
  set distance-group 10]
  ask n-of round (0.057 * count ICEVs with [another-city? = true and goal-group = 2]) (ICEVs with [distance-group = 200])
 [set highway-distance random (100 - 91 + 1) + 91
  set distance-group 9]
 ask n-of round (0.0665 * count ICEVs with [another-city? = true and goal-group = 2]) (ICEVs with [distance-group = 200])
 [set highway-distance random (90 - 81 + 1) + 81
  set distance-group 8]
 ask n-of round (0.076 * count ICEVs with [another-city? = true and goal-group = 2]) (ICEVs with [distance-group = 200])
 [set highway-distance random (80 - 71 + 1) + 71
  set distance-group 7]
 ask n-of round (0.0855 * count ICEVs with [another-city? = true and goal-group = 2]) (ICEVs with [distance-group = 200])
 [set highway-distance random (70 - 61 + 1) + 61
  set distance-group 6]
 ask n-of round (0.095 * count ICEVs with [another-city? = true and goal-group = 2]) (ICEVs with [distance-group = 200])
 [set highway-distance random (60 - 51 + 1) + 51
  set distance-group 5]
 ask n-of round (0.1045 * count ICEVs with [another-city? = true and goal-group = 2]) (ICEVs with [distance-group = 200])
 [set highway-distance random (50 - 41 + 1) + 41
  set distance-group 4]
 ask n-of round (0.114 * count ICEVs with [another-city? = true and goal-group = 2]) (ICEVs with [distance-group = 200])
 [set highway-distance random (40 - 31 + 1) + 31
  set distance-group 3]
 ask n-of round (0.1235 * count ICEVs with [another-city? = true and goal-group = 2]) (ICEVs with [distance-group = 200])
 [set highway-distance random (30 - 21 + 1) + 21
  set distance-group 2]
 ask ICEVs with [distance-group = 200] ;; rest is about 13.55%, to make sure there is no ICEV left with group 100
 [set highway-distance random (20 - 10 + 1) + 10
  set distance-group 1]
end

;;;;;;;;;;;;;
;; RUNTIME ;;
;;;;;;;;;;;;;

to go
   ask ICEVs
   [

     face next-patch-ICEV   ;; heading to its goal (patch closest to its goal)

     if ((member? patch-here [ neighbors4 ] of house) = true) ;; if I am in my house or going through
      [if two-goals? = false
        [ifelse (worked-today? = true and another-city? = false) or (worked-today? = true and another-city? = true and highway-done? = true)
           [fd 0  ;; if I have already been to work1 today and I am back home -> stay until next day
            set home-time-ticks-ICEV home-time-ticks-ICEV + 1] ;; count home-ticks
           [continue-ICEV]
         ]
      if two-goals? = true
        [ifelse (worked2-today? = true and another-city? = false) or (worked2-today? = true and another-city? = true and highway-done? = true)
           [fd 0  ;; if I have already been to work1 today and I am back home -> stay until next day
            set home-time-ticks-ICEV home-time-ticks-ICEV + 1] ;; count home-ticks
           [continue-ICEV]
          ]
       ]

     if ((member? patch-here [ neighbors4 ] of house) = false) and (pcolor = white or pcolor = brown or pcolor = blue or pcolor = violet)  ;; when on a road
     [continue-ICEV]  ;; moving forward by city speed (50km/h) in the city
     if ((pcolor = grey))  ;; when on a highway
     [go-fast-ICEV]  ;; moving forward by higher speed (100km/h) on a highway
     ]


  ask EVs
  [
    face next-patch-EV

     if ((member? patch-here [ neighbors4 ] of house) = true) ;; if I am in my house or going through
    [
     if two-goals? = false
            [ifelse ((another-city? = true and worked-today? = true and big-charged? = true) or (another-city? = false and worked-today? = true))
                 [set got-home? true ;; only if I am done with my trip
                  ifelse home-charger? = true
                     [home-charge]  ;; charge at home
                     [if energy <= 0  ;; if teleported because deployed battery
                       [
                         set invested invested + (50 * 0.492 * Paid-electricity-share)
                         set energy 50
                        ]
                      fd 0
                      set home-time-ticks-EV home-time-ticks-EV + 1]] ;; stay at home only after whole day journey is done this day, count home-ticks
                 [continue-EV]
             ]


     if two-goals? = true
           [ifelse ((another-city? = true and worked2-today? = true and big-charged? = true) or (another-city? = false and worked2-today? = true))
                  [set got-home? true ;; only if I am done with my trip
                   ifelse home-charger? = true
                     [home-charge]  ;; charge at home
                   [if energy <= 0  ;; if teleported because deployed battery
                       [
                         set invested invested + (50 * 0.492 * Paid-electricity-share)
                         set energy 50
                        ]
                     fd 0
                     set home-time-ticks-EV home-time-ticks-EV + 1]]   ;; stay at home only after whole day journey is done this day, count home-ticks
                 [continue-EV ]
             ]
    ]


    if ((member? patch-here [ neighbors4 ] of house) = false) and (energy > 0) and (pcolor = white or pcolor = brown)  ;; if enough energy and on the road in the city and not on the charger or in house
     [continue-EV]   ;; moving forward by city speed (50km/h) in the city


    if pcolor = blue
      [ifelse need-charge? = true
        [charge]   ;; ;; if on a public blue charger and need chargstay and charge until full
        [continue-EV] ];; if on a public blue charger and enough energy for trip


    if ((energy > 0) and  (pcolor = grey) and count-down < ( highway-distance / 2 ))  ;; if enough energy, I am on a highway in the first half of the distance between 2 cities
    [go-fast-EV-1]  ;; move with 100km/h speed


    if ((energy > 0) and  (pcolor = grey) and count-down >= ( highway-distance / 2 ))  ;; if enough energy, I am on a highway in the second half of the distance between 2 cities (after recharge on a violet patch)
    [go-fast-EV-2]  ;; move with 100km/h speed


    if pcolor = violet  ;; when on a highway charger
    [big-charge]  ;; recharge on a highway

    if energy <= 0
    [ set goal house
      set worked-today? true
      if another-city? = true
        [set big-charged? true]
      if two-goals? = true
        [set worked2-today? true]
      move-to one-of (([neighbors4] of house) with [pcolor = white])  ;; assistance service if no energy

     ]
  ]



tick
  if ( ticks / 1200 ) - floor ( ticks / 1200 ) = 0 ;; if the day is over
  [ set EVs-not-get-home EVs-not-get-home + (count EVs with [got-home? = false])  ;; count failures of EVs which did not manage to get home
    set Avg-EVs-not-get-home-a-day ((EVs-not-get-home * 1200) / ticks)
    ask EVs
    [
      set invested (invested + 22.466)  ;; add daily maintenance
      if got-home? = true  ;; if not got-home, it will get there and wait until the next day
      [
        set worked-today? false  ;; I have to go to work1 next day after my stay in house last day
        set worked2-today? false  ;; I have to go to work2 next day after my stay in house last day
        set big-charged? false  ;; for the highway EV
        set got-home? false  ;; setup for next day
      ]
     ]
   ask ICEVs
   [
     set invested (invested + 32.329)  ;; add daily maintenance
     set worked-today? false  ;; I have to go to work1 next day after my stay in house last day
     set worked2-today? false ;; I have to go to work2 next day after my stay in house last day
     set highway-done? false  ;; for the highway ICEV
   ]
   ask ICEVs with [another-city? = true]
     [set invested invested + (Highway-toll-ICEV / 365)]  ;; add tolls for highway
   ask EVs with [another-city? = true]
     [set invested invested + (Highway-toll-EV / 365)]
  ]



set EVs-with-0-energy EVs-with-0-energy + (count EVs with [energy <= 0])
set Avg-km-made-EV-a-day (sum [km-made-by-EV] of EVs) * 1200 / ((count EVs) * ticks)
set Max-km-made-EV-a-day max [(km-made-by-EV * 1200) / ticks] of EVs
set Min-km-made-EV-a-day min [(km-made-by-EV * 1200) / ticks] of EVs
set Avg-km-made-ICEV-a-day (sum [km-made-by-ICEV] of ICEVs) * 1200 / ((count ICEVs) * ticks)
set Max-km-made-ICEV-a-day max [(km-made-by-ICEV * 1200) / ticks] of ICEVs
set Min-km-made-ICEV-a-day min [(km-made-by-ICEV * 1200) / ticks] of ICEVs
set Ticks-for-one-km-made-EV ((sum [total-active-ticks-EV + total-charging-ticks] of EVs) / sum [km-made-by-EV] of EVs)
set Ticks-for-one-km-made-ICEV ((sum [total-active-ticks-ICEV] of ICEVs) / sum [km-made-by-ICEV] of ICEVs)
set CZK-invested-for-1-km-EV ((sum [invested] of EVs) / sum [km-made-by-EV] of EVs)
set CZK-invested-for-1-km-ICEV ((sum [invested] of ICEVs) / sum [km-made-by-ICEV] of ICEVs)
set CZK-invested-for-1-day-EV (((sum [invested] of EVs) * 1200) / (count EVs * ticks))
set CZK-invested-for-1-day-ICEV (((sum [invested] of ICEVs) * 1200) / (count ICEVs * ticks))
set Avg-hours-at-home-a-day-ICEV ((24 * sum [Home-time-ticks-ICEV] of ICEVs)/(count ICEVs * ticks))
set Min-hours-at-home-a-day-ICEV min [Home-time-ticks-ICEV   * 24 / ticks] of ICEVs
set Max-hours-at-home-a-day-ICEV max [Home-time-ticks-ICEV * 24 / ticks] of ICEVs
set Avg-hours-at-home-a-day-EV ((24 * sum [Home-time-ticks-EV] of EVs)/(count EVs * ticks))
set Min-hours-at-home-a-day-EV min [Home-time-ticks-EV * 24 / ticks] of EVs
set Max-hours-at-home-a-day-EV max [Home-time-ticks-EV * 24 / ticks] of EVs
set Km-made-by-EV-total (sum [km-made-by-EV] of EVs / count EVs)
set Km-made-by-ICEV-total (sum [km-made-by-ICEV] of ICEVs / count ICEVs)
set Avg-active-hours-EV-a-day (sum [total-active-ticks-EV] of EVs) * 24 / ((count ICEVs) * ticks)
set Max-active-hours-EV-a-day max [total-active-ticks-EV * 24 / ticks] of EVs
;; monitor observed variables of interest


display-labels   ;; to display energy for EVs

end

;;;;;;;;;;;;;;;;;;
;;RUNTIME others;;
;;;;;;;;;;;;;;;;;;

to-report next-patch-ICEV


  if goal = house and (member? patch-here [ neighbors4 ] of house) ;; if I am going home and I am next to the patch that is my home
  [ set count-down 0  ;; reset counter for highway
    set goal work1  ;; my goal gets set to the patch that is my work1
    if another-city? = true ;; if I am comuting
    [set goal highway]  ;; I have to go first on a highway then to my goal
  ]

    if goal = work1 and (member? patch-here [ neighbors4 ] of work1) ;; if I am going to work1 and I am next to the patch that is my work1
  [
    set count-down 0  ;; reset counter for highway
    set invested (invested + Parking-fee-ICEV)
    set worked-today? true  ;; I Have already been to work1 today -> I will stay at home until the end of day
    set highway-done? false  ;; to reset for highway
    set goal house
    if two-goals? = true  ;; If I have to also to work2
    [set goal work2]  ;; go to work2
    if two-goals? = false and another-city? = true ;; if I am comuting and not to work2 -> highway
    [ set goal highway]  ;; I have to go first on a highway
  ]

  if goal = work2 and (member? patch-here [ neighbors4 ] of work2) ;; if I am going to work2 and I am next to the patch that is my work2
  [ set worked2-today? true  ;; I Have already been to work1 today -> I will stay at home until the end of day
    set goal house  ;; my goal gets set to the patch that is my home
    if another-city? = true ;; if I am comuting
    [set goal highway]  ;; I have to go first on a highway then to my goal
    ]

  if goal = highway and patch-here = highway and worked-today? = false
  [
  set goal work1
  ]  ;; I am on a highway on way to work

    if goal = highway and patch-here = highway and worked-today? = true
  [
  set goal house
   ]  ;; I am on a hughway from the way back from work to house

  ifelse (member? patch-here [ neighbors4 ] of house) or (member? patch-here [ neighbors4 ] of work1) or (member? patch-here [ neighbors4 ] of work2) or (patch-here = highway) ;; CHOICES is an agentset of the candidate patches that the car can move to (white patches are roads)
  [set choices neighbors with [ pcolor = white or pcolor = blue or pcolor = grey]] ;; if going from house or work1 or work2
  [set choices neighbors with [ pcolor = white or pcolor = blue or pcolor = grey and self != [patch-ahead -1] of myself]]  ;; if not going from house/work1, to prevent locked moving between 2 patches
  set choice min-one-of choices [ distance [goal] of myself ]  ;; choose the patch closest to the goal, this is the patch the car will move to
  report choice  ;; report the chosen patch

end ;; Taken from Traffic Grid Goal model




to-report next-patch-EV



  if goal = house and (member? patch-here [ neighbors4 ] of house) ;; if I am going home and I am next to the patch that is my home
  [ set count-down 0  ;; setup for next highway trip
    set goal work1  ;; my goal gets set to the patch that is my home
    set trip-distance
    (distance (patch ([pxcor] of goal) ([pycor] of patch-here)))
    + (distance (patch ([pxcor] of patch-here) ([pycor] of goal))) ;; distance to next goal to determine if enough energy for it -> to determine need-charge?

    if another-city? = true ;; if I am comuting
    [set goal highway  ;; I have to go first on a highway then to my goal
    set trip-distance
    (distance (patch ([pxcor] of goal) ([pycor] of patch-here)))
    + (distance (patch ([pxcor] of patch-here) ([pycor] of goal))) ;; recalculate because different goal
    + (highway-distance / 2 )] ;; need to add distance to the highway charger to determine if enough energy for it -> to determine need-charge?

    ifelse trip-distance > (energy - 20)   ;; calculate if I have enough energy for this part-trip
   [set need-charge? true
    set goal min-one-of chargers [distance myself]]  ;; I have to charge during my trip
   [set need-charge? false]  ;; I dont have to charge
    ]



  if goal = work1 and (member? patch-here [ neighbors4 ] of work1) ;; if I am going to work1 and I am next to the patch that is my work1
  [ set count-down 0   ;; setup for next highway trip
    set invested (invested + Parking-fee-EV)
    set worked-today? true  ;; I Have already been to work1 today -> I will stay at home until the end of day
    set big-charged? false  ;; in order to go to charger on way back on a highway
    set goal house  ;; go to house
    set trip-distance
    (distance (patch ([pxcor] of goal) ([pycor] of patch-here)))
    + (distance (patch ([pxcor] of patch-here) ([pycor] of goal))) ;; distance to next goal to determine if enough energy for it -> to determine need-charge?

   if two-goals? = false and another-city? = true ;; if I am comuting and not to work2
    [set goal highway  ;; I have to go first on a highway then to my goal
    set trip-distance
    (distance (patch ([pxcor] of goal) ([pycor] of patch-here)))
    + (distance (patch ([pxcor] of patch-here) ([pycor] of goal)))
    + (highway-distance / 2 )] ;; distance to the highway charger to determine if enough energy for it -> to determine need-charge?

    if two-goals? = true  ;; if I have work2
    [set goal work2  ;; I have to go first on a highway then to my goal
    set trip-distance
    (distance (patch ([pxcor] of goal) ([pycor] of patch-here)))
    + (distance (patch ([pxcor] of patch-here) ([pycor] of goal)))] ;; recalculate distance to next goal to determine if enough energy for it -> to determine need-charge?


    ifelse trip-distance > (energy - 20)  ;; calculate if I have enough energy for this part-trip plus reserve
   [set need-charge? true
   set goal min-one-of chargers [distance myself]]  ;; I have to charge during my trip
    [set need-charge? false]  ;; I dont have to charge
  ]



  if goal = work2 and (member? patch-here [ neighbors4 ] of work2) ;; if I am going to work2 and I am next to the patch that is my work2
  [
    set worked2-today? true
    set goal house  ;; my goal gets set to the patch that is my home
    set trip-distance
    (distance (patch ([pxcor] of goal) ([pycor] of patch-here)))
    + (distance (patch ([pxcor] of patch-here) ([pycor] of goal))) ;; distance to next goal to determine if enough energy for it -> to determine need-charge?

    if another-city? = true ;; if I am comuting and not to work2
    [set goal highway  ;; I have to go first on a highway then to my goal
    set trip-distance
    (distance (patch ([pxcor] of goal) ([pycor] of patch-here)))
    + (distance (patch ([pxcor] of patch-here) ([pycor] of goal)))
    + (highway-distance / 2 )] ;; distance to the highway charger to determine if enough energy for it -> to determine need-charge?


   ifelse trip-distance > (energy - 20)  ;; calculate if I have enough energy for this part-trip
   [set need-charge? true
   set goal min-one-of chargers [distance myself]]  ;; I have to charge during my trip
   [set need-charge? false]  ;; I dont have to charge
  ]



  if goal = highway and patch-here = highway and worked-today? = true ;; on way back from work
  [
  set goal house
   ]


  if goal = highway and patch-here = highway and worked-today? = false  ;; on way to work
  [
  set goal work1
  ]


  ifelse (member? patch-here [ neighbors4 ] of house) or (member? patch-here [ neighbors4 ] of work1) or (member? patch-here [ neighbors4 ] of work2) or (patch-here = highway) ;; CHOICES is an agentset of the candidate patches that the car can move to (white patches are roads)
  [set choices neighbors with [ pcolor = white or pcolor = blue or pcolor = grey]] ;; if going from house or work1 or work2
  [set choices neighbors with [ pcolor = white or pcolor = blue or pcolor = grey and self != [patch-ahead -1] of myself]]  ;; if not going from house/work1, to prevent locked moving between 2 patches
  set choice min-one-of choices [ distance [goal] of myself ]  ;; choose the patch closest to the goal, this is the patch the car will move to
  report choice  ;; report the chosen patch

end ;; Taken from Traffic Grid Goal model





;;;;ICEV;;;;


to continue-ICEV  ;; moving in the city ICEV
     fd (1 * (City-speed-ICEV / 50))  ;; moving with cons. speed, 50km/h
     set invested invested + (1.643 * (City-speed-ICEV / 50))   ;; invested money based on price of fuel for 1 km
     set km-made-by-ICEV km-made-by-ICEV + (City-speed-ICEV / 50)  ;; total km made by all ICEVs increases
     set total-active-ticks-ICEV total-active-ticks-ICEV + 1  ;; total active time for ICEV to determine time cost for one km
end

to go-fast-ICEV ;; ;; moving with 100km/h speed on a highway
  set invested invested + 3.286 ;; more km made -> more money for fuel
  set km-made-by-ICEV km-made-by-ICEV + 2 ;; adding km to monitor, higher speed
  set total-active-ticks-ICEV total-active-ticks-ICEV + 1  ;; total active time for ICEVs to determine time cost for one km
  set count-down count-down + 2  ;; moving with higher speed, distance increment
  if count-down >= highway-distance  ;; if in the end of highway
  [ set highway-done? true
    continue-ICEV]  ;; move in a city
end


;;;;EV;;;;



to continue-EV  ;; movig in the city EV
    fd (1 * (City-speed-EV / 50))  ;; move 1 km, 50km/h
    set energy energy - (City-speed-EV / 50)  ;; loosing energy (electricity)
    set invested invested + (0.492 * Paid-electricity-share * (City-speed-EV / 50)) ;; money invested for electricity based on its price for 1 km
    set km-made-by-EV km-made-by-EV + (1 * (City-speed-EV / 50))  ;; ;; total km made by all EVs increases
    set total-active-ticks-EV total-active-ticks-EV + 1  ;; total active time for EV to determine time cost for one km
end

to charge   ;; only for EVs standing on a charger with not full battery
  set energy energy + 0.76  ;; 1 km added with one tick
  set total-charging-ticks total-charging-ticks + 1  ;; for calculating time cost for EVs

;; if I am only going home and have home charger, it is enough to charge for the trip-distance only
  if (home-charger? = true) and (worked-today? = true) and (another-city? = false or (another-city? = true and big-charged? = true)) and (two-goals? = false or (two-goals? = true and worked2-today? = true))
      [set goal house
       set trip-distance
      ((distance (patch ([pxcor] of goal) ([pycor] of patch-here)))
     + (distance (patch ([pxcor] of patch-here) ([pycor] of goal))))
       if energy >= trip-distance + 20
          [set need-charge? false
           continue-EV]
       ]

;; for other cases
  if (energy >= 164)
  [
      if worked-today? = false
        [if another-city? = false
          [set goal work1]
         if another-city? = true
          [ifelse big-charged? = false
            [set goal highway]
            [set goal work1]
          ]
         ]

      if worked-today? = true and two-goals? = false
          [if another-city? = false
             [set goal house]
           if another-city? = true
             [ifelse big-charged? = false
                [set goal highway]
                [set goal house]
              ]
          ]

      if worked-today? = true and two-goals? = true
          [if worked2-today? = false
              [set goal work2]
           if worked2-today? = true
              [if another-city? = false
                 [set goal house]
               if another-city? = true
                 [ifelse big-charged? = false
                    [set goal highway]
                    [set goal house]
                 ]
              ]
          ]
  ;; to set my previous goals because it changed to charger and I have to change it back

      set trip-distance
      (distance (patch ([pxcor] of goal) ([pycor] of patch-here)))
     + (distance (patch ([pxcor] of patch-here) ([pycor] of goal)))

      if goal = highway
      [set trip-distance
        trip-distance + (highway-distance / 2 )]  ;; need to add if going through highway

    ifelse trip-distance > (energy - 20)  ;; calculate if I have enough energy for this part-trip
    [set need-charge? true]
    [set need-charge? false]  ;; I dont have to charge

    continue-EV ;; when full battery -> go
  ]
end

to home-charge   ;; only for EVs in a house with not full battery and with home-charger? = true
   if energy < 164
   [
   set energy energy + 0.35  ;; 1 km added with one tick
   set home-time-ticks-EV home-time-ticks-EV + 1 ;; stay at home only after whole day journey is done this day, count home-ticks
   set total-charging-ticks total-charging-ticks + 1] ;; for calculating time cost for EVs
   if (((energy >= 164) and two-goals? = true and (worked2-today? = true))  ;; when full battery and been in work1 today and I dont have work2 -> stay until next day
       or ((energy >= 164) and two-goals? = false and (worked-today? = true)))
           [fd 0
            set home-time-ticks-EV home-time-ticks-EV + 1]
end

to go-fast-EV-1  ;; moving with 100km/h speed on first half of the highway
  set energy energy - 2 ;; faster moving -> more energy needed
  set invested invested + (0.984 * Paid-electricity-share)  ;; more km made -> more money for electricity
  set km-made-by-EV km-made-by-EV + 2  ;; adding km to monitor, higher speed
  set total-active-ticks-EV total-active-ticks-EV + 1  ;; total active time for EV to determine time cost for one km
  set count-down count-down + 2  ;; moving with higher speed, distance increment
  if count-down >= ( highway-distance / 2 )  ;; if in the middle of a highway
   [set trip-distance
       (distance (patch ([pxcor] of goal) ([pycor] of highway)))
       + (distance (patch ([pxcor] of highway) ([pycor] of goal)))
       + (highway-distance / 2 )  ;; distance to the goal + second half of highway from chargerset count-down count-down + 1
    move-to patch (world-size - 1 ) ((world-size * -1) + 1)]  ;; move to the violet charger on a highway to charge
 end



to go-fast-EV-2  ;; moving with 100km/h speed on second half of the highway
  set energy energy - 2 ;; faster moving -> more energy needed
  set invested invested + (0.984 * Paid-electricity-share)  ;; more km made -> more money for electricity
  set km-made-by-EV km-made-by-EV + 2  ;; adding km to monitor, higher speed
  set total-active-ticks-EV total-active-ticks-EV + 1  ;; total active time for EVs to determine time cost for one km
  set count-down count-down + 2  ;; moving with higher speed, distance increment
  if count-down >= ( highway-distance )  ;; if in the end of highway
  [continue-EV]  ;; move in a city
end



to big-charge  ;; for EVs standing on a highway violet charger
  set energy energy + 0.76  ;; charge 1km with 1 tick
  set total-charging-ticks total-charging-ticks + 1  ;; for calculating time cost for EVs

  ;; if I am only going home and have home charger, it is enough to charge for trip-distance only
  if (energy >= (trip-distance + 20)) and (home-charger? = true) and (worked-today? = true) and (two-goals? = false or (two-goals? = true and worked2-today? = true))
      [
       set big-charged? true
       move-to patch (world-size) (world-size * -1) ;; when full battery -> go back to the highway -> second half of the highway
       set need-charge? false  ;; I dont have to charge
      ]

 ;; other cases
  if (energy >= 164)
  [
   set big-charged? true
   move-to patch (world-size) (world-size * -1) ;; when full battery -> go back to the highway -> second half of the highway

   ifelse trip-distance > (energy - 20)  ;; calculate if I have enough energy for this part-trip to work1/house
   [set need-charge? true
   set goal min-one-of chargers [distance myself]]  ;; I have to charge during my trip
   [set need-charge? false]  ;; I dont have to charge
  ]
end



to display-labels  ;; display energy left for EVs
  ask EVs [ set label (round energy) ]
end
@#$#@#$#@
GRAPHICS-WINDOW
926
22
1337
434
-1
-1
13.0
1
10
1
1
1
0
0
0
1
-15
15
-15
15
0
0
1
ticks
30.0

BUTTON
13
19
77
52
Setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
90
19
153
52
Go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
722
232
894
265
number-of-chargers
number-of-chargers
1
100
30.0
1
1
NIL
HORIZONTAL

SLIDER
722
23
915
56
price-ICEVs
price-ICEVs
0
2000000
586000.0
1000
1
CZK
HORIZONTAL

SLIDER
724
59
916
92
price-EVs
price-EVs
0
2000000
878000.0
1000
1
CZK
HORIZONTAL

SLIDER
22
61
194
94
world-size
world-size
1
100
15.0
1
1
km
HORIZONTAL

SLIDER
24
136
196
169
EV-market-share
EV-market-share
0
1
0.5
0.1
1
NIL
HORIZONTAL

SLIDER
23
99
195
132
number-of-cars
number-of-cars
0
1000
500.0
100
1
NIL
HORIZONTAL

MONITOR
525
308
695
353
CZK-invested-for-1-km-ICEV
CZK-invested-for-1-km-ICEV
2
1
11

MONITOR
524
257
690
302
CZK-invested-for-1-km-EV
CZK-invested-for-1-km-EV
2
1
11

MONITOR
516
143
680
188
Ticks-for-one-km-made-EV
Ticks-for-one-km-made-EV
3
1
11

SLIDER
6
174
261
207
Probability-of-going-to-another-city-1-goal
Probability-of-going-to-another-city-1-goal
0
1
0.5
0.1
1
NIL
HORIZONTAL

SLIDER
3
322
274
355
max-distance-between-two-cities-EV
max-distance-between-two-cities-EV
10
370
200.0
1
1
km
HORIZONTAL

MONITOR
501
192
706
237
Ticks-for-one-km-made-ICEV
Ticks-for-one-km-made-ICEV
3
1
11

MONITOR
315
25
372
70
Days
ticks / 1200
3
1
11

SLIDER
10
394
236
427
Probability-of-having-2-goals-EV
Probability-of-having-2-goals-EV
0
1
0.5
0.1
1
NIL
HORIZONTAL

SLIDER
718
198
895
231
Paid-electricity-share
Paid-electricity-share
0
1
1.0
0.1
1
NIL
HORIZONTAL

MONITOR
254
25
311
70
Hours
Ticks / 50
2
1
11

MONITOR
518
25
662
70
Avg-km-made-EV-a-day
Avg-km-made-EV-a-day
1
1
11

MONITOR
517
74
674
119
Avg-km-made-ICEV-a-day
Avg-km-made-ICEV-a-day
1
1
11

SLIDER
725
142
897
175
City-speed-ICEV
City-speed-ICEV
0
100
50.0
1
1
NIL
HORIZONTAL

SLIDER
725
105
897
138
City-speed-EV
City-speed-EV
0
100
50.0
1
1
NIL
HORIZONTAL

SLIDER
730
319
911
352
Highway-toll-ICEV
Highway-toll-ICEV
0
4000
1500.0
10
1
CZK
HORIZONTAL

SLIDER
729
353
901
386
Highway-toll-EV
Highway-toll-EV
0
4000
1500.0
10
1
CZK
HORIZONTAL

MONITOR
296
123
473
168
Avg-hours-at-home-a-day-EV
Avg-hours-at-home-a-day-EV
3
1
11

MONITOR
270
74
495
119
Avg-hours-at-home-a-day-ICEV
Avg-hours-at-home-a-day-ICEV
3
1
11

MONITOR
442
26
499
71
Years
ticks / 432000
3
1
11

MONITOR
377
25
434
70
Months
(ticks / 36000)
3
1
11

SLIDER
2
360
286
393
max-distance-between-two-cities-ICEV
max-distance-between-two-cities-ICEV
10
370
370.0
1
1
km
HORIZONTAL

SLIDER
6
211
259
244
Probability-of-going-to-another-city-2-goals
Probability-of-going-to-another-city-2-goals
0
1
0.4
0.1
1
NIL
HORIZONTAL

MONITOR
704
591
820
636
EVs-with-0-energy
EVs-with-0-energy
0
1
11

MONITOR
704
644
825
689
EVs-not-get-home
EVs-not-get-home
0
1
11

SLIDER
13
248
221
281
Probability-of-home-charging
Probability-of-home-charging
0
1
0.9
0.1
1
NIL
HORIZONTAL

SLIDER
9
429
248
462
Probability-of-having-2-goals-ICEV
Probability-of-having-2-goals-ICEV
0
1
0.5
0.1
1
NIL
HORIZONTAL

MONITOR
683
536
860
581
Avg-EVs-not-get-home-a-day
Avg-EVs-not-get-home-a-day
3
1
11

SLIDER
727
406
899
439
Parking-fee-EV
Parking-fee-EV
0
100
0.0
1
1
NIL
HORIZONTAL

MONITOR
9
538
187
583
NIL
Max-hours-at-home-a-day-EV
10
1
11

MONITOR
10
593
200
638
NIL
Max-hours-at-home-a-day-ICEV
3
1
11

MONITOR
229
537
402
582
NIL
Min-hours-at-home-a-day-EV
3
1
11

MONITOR
231
592
417
637
NIL
Min-hours-at-home-a-day-ICEV
3
1
11

MONITOR
312
231
458
276
NIL
Max-km-made-EV-a-day
17
1
11

MONITOR
310
341
468
386
NIL
Max-km-made-ICEV-a-day
17
1
11

MONITOR
314
183
455
228
NIL
Min-km-made-EV-a-day
17
1
11

MONITOR
314
291
472
336
NIL
Min-km-made-ICEV-a-day
17
1
11

MONITOR
324
392
455
437
NIL
Km-made-by-EV-total
3
1
11

MONITOR
322
439
465
484
NIL
Km-made-by-ICEV-total
3
1
11

MONITOR
462
537
626
582
NIL
Avg-active-hours-EV-a-day
17
1
11

MONITOR
464
594
629
639
NIL
Max-active-hours-EV-a-day
17
1
11

MONITOR
523
359
686
404
NIL
CZK-invested-for-1-day-EV
3
1
11

MONITOR
524
412
699
457
NIL
CZK-invested-for-1-day-ICEV
3
1
11

SLIDER
727
444
899
477
Parking-fee-ICEV
Parking-fee-ICEV
0
100
0.0
1
1
NIL
HORIZONTAL

@#$#@#$#@
## WHAT IS IT?

This agent-based model of a vehicle traffic in the Czech Republic was developed for the purposes of a Bachelor thesis to explore the effect of various policies on eletric vehicles' costs, usefulness and market potential.


## CREDITS AND REFERENCES

WOJNAROVÁ, Renáta. Agent-Based Analysis of Market Potential for Electric Vehicles 
in the Czech Republic. Prague 2017. 66 pp. Bachelor thesis (Bc.)
Charles University, Faculty of Social Sciences, Institute of Economic Studies. 
Thesis supervisor PhDr. Jiří Kukačka, Ph.D.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
